//
// Connection.java - Handle connections we listened for
//

//...simports:0:
import java.net.*;
import java.io.*;
import java.util.*;
//...e

public class Connection extends Thread
  {
  protected Socket   s;
  protected int      connId;
  protected Listener l;
  protected int      buffsz;
  // Called by Listener
//...sConnection:2:
public Connection(Socket s, int connId, Listener l, int buffsz)
  {
  this.s      = s;
  this.connId = connId;
  this.l      = l;
  this.buffsz = buffsz;
  }
//...e
//...sforward:2:
public void forward()
  {
  l.send("a", connId, null, 0);
  }
//...e
//...shandle:2:
public boolean handle(String op, byte[] b)
  {
  if ( op.equals("A") )
    this.start();
  else if ( op.startsWith("A") )
    {
    l.error("Connection terminated connId="+connId);
    try
      {
      s.close();
      }
    catch ( IOException e )
      {
      l.error("Couldn't close socket for connId="+connId);
      }
    l.connectionEnded(connId);
      // Connection isn't running as a thread, so we can't rely on the
      // run() method invoking l.connectionEnded(), so do it here.
    }
  else if ( op.equals("D") )
    try
      {
      OutputStream os = s.getOutputStream();
      os.write(b);
      os.flush();
      }
    catch ( IOException e2 )
      {
      try
        {
        l.error("Couldn't write data to socket for connId="+connId);
        s.close();
        }
      catch ( IOException e3 )
        {
        l.error("Couldn't close socket for connId="+connId);
        }
      }
  else if ( op.equals("C") )
    try
      {
      s.close();
      }
    catch ( IOException e4 )
      {
      l.error("Couldn't close socket on request for connId="+connId);
      }
  else
    return false;
  return true;
  }
//...e
  // Side effect of this.start() above
//...srun:2:
public void run()
  {
  try
    {
    InputStream is = s.getInputStream();
    byte[] b = new byte[buffsz];
    int n;
    while ( (n = is.read(b)) > 0 )
      l.send("d", connId, b, n);
    is.close();
    l.send("c", connId, null, 0);
    }
  catch ( IOException e )
    // We can get here legitimately if we received a C from the other end
    {
    }
  finally
    {
    l.connectionEnded(connId);
    }
  }
//...e
  }
