//
// Proxy.java - Respond to packets received over the Textflow interface
//

//...simports:0:
import java.net.*;
import java.io.*;
import java.util.*;
//...e

public class Proxy
  {
  protected MapPortHostPort map;
  protected Textflow tf;
  protected int buffsz;
  protected Dictionary<String,ProxyConnection> pconns = new Hashtable<String,ProxyConnection>();
  // Helpers
//...skeyOf:2:
protected static String keyOf(int port, int connId)
  {
  return Integer.toString(port)+"/"+Integer.toString(connId);
  }
//...e
  // Called from Tunnel.main
//...sProxy:2:
public Proxy(MapPortHostPort map, Textflow tf, int buffsz)
  {
  this.map    = map;
  this.tf     = tf;
  this.buffsz = buffsz;
  }
//...e
  // Called from Textflow
//...shandle:2:
public boolean handle(String op, int port, int connId, byte[] b)
  {
  if ( op.equals("a") )
    // new Connection accepted
    {
    ProxyConnection pconn = new ProxyConnection(port, connId, map, tf, this, buffsz);
    pconns.put(keyOf(port, connId), pconn);
    pconn.start();
    return true;
    }
  else
    {
    ProxyConnection pconn = pconns.get(keyOf(port, connId));
    if ( pconn != null )
      return pconn.handle(op, b);
    else
      // This can happen legitimately if the server closes the connection
      // and the client still has stuff to say to it, such as more data
      // or the fact it wants to close the connection itself.
      // We'll assume it was correct and we would have handled it.
      return true;
    }
  }
//...e
  // Called from ProxyConnection
//...sconnectionEnded:2:
public void connectionEnded(int port, int connId)
  {
  pconns.remove(keyOf(port, connId));
  }
//...e
  }
