//
// ProxyConnection.java - Proxy for connection originating from other side
//

//...simports:0:
import java.net.*;
import java.io.*;
import java.util.*;
//...e

class ProxyConnection extends Thread
  {
  protected int             port;
  protected int             connId;
  protected MapPortHostPort map;
  protected Textflow        tf;
  protected Proxy           prx;
  protected Socket          s;
  protected int             buffsz;
//...sProxyConnection:2:
public ProxyConnection(int port, int connId, MapPortHostPort map, Textflow tf, Proxy prx, int buffsz)
  {
  this.port   = port;
  this.connId = connId;
  this.map    = map;
  this.tf     = tf;
  this.prx    = prx;
  this.buffsz = buffsz;
  }
//...e
//...srun:2:
public void run()
  {
  // Making the connection
  HostPort hp = map.get(new Integer(port));
  try
    {
    s = new Socket(hp.host, hp.port);
    tf.send("A", port, connId, null, 0); // ProxyConnection has connected
    }
  catch ( UnknownHostException e )
    {
    tf.error("Couldn't resolve hostname in "+hp.host+":"+hp.port+" on behalf of port="+port+" connId="+connId);
    tf.send("Auhe", port, connId, null, 0);
    prx.connectionEnded(port, connId);
    return;
    }
  catch ( IOException e )
    {
    tf.error("Couldn't connect to "+hp.host+":"+hp.port+" on behalf of port="+port+" connId="+connId);
    tf.send("Aioe", port, connId, null, 0);
    prx.connectionEnded(port, connId);
    return;
    }
  // Data handling
  try
    {
    InputStream is = s.getInputStream();
    byte[] b = new byte[buffsz];
    int n;
    while ( (n = is.read(b)) > 0 )
      tf.send("D", port, connId, b, n);
    is.close();
    tf.send("C", port, connId, null, 0);
    }
  catch ( IOException e )
    // Can legitimately happen
    // Typically seen when the client drops the connection
    // eg: Firefox receives an SSL certificate it doesn't trust
    {
    }
  finally
    {
    prx.connectionEnded(port, connId);
    }
  }
//...e
  // Called by Proxy
//...shandle:2:
public boolean handle(String op, byte[] b)
  {
  if ( op.equals("d") )
    try
      {
      OutputStream os = s.getOutputStream();
      os.write(b);
      os.flush();
      }
    catch ( IOException e )
      {
      // This can legitimately happen if the server side closes the connection
      // prior to the client having sent it all the data it might want to send.
      try
        {
        s.close();
        }
      catch ( IOException e2 )
        {
        tf.error("Couldn't close proxy socket relating to port="+port+" connId="+connId);
        }
      }
  else if ( op.equals("c") )
    try
      {
      s.close();
      }
    catch ( IOException e3 )
      {
      tf.error("Couldn't close proxy socket relating to port="+port+" connId="+connId);
      }
  else
    return false;
  return true;
  }
//...e
  }
