//
// Tunnel.java - Tunnel over console channel
//

//...simports:0:
import java.net.*;
import java.io.*;
import java.util.*;
//...e

public class Tunnel
  {
//...sfatal:2:
protected static void fatal(String e)
  {
  System.err.println("Tunnel: "+e);
  System.exit(1);
  }
//...e
//...susage:2:
protected static void usage()
  {
  System.out.println("usage: Tunnel {-L lp:rh:rp} {-R rp:lh:lp} [-j java] [-t jar] [-b size] command");
  System.out.println("flags: -L lp:rh:rp  port forward");
  System.out.println("       -R rp:lh:lp  port backward");
  System.out.println("       -j java      remote java command (default: java)");
  System.out.println("       -t jar       remote .jar file (default: tunnel.jar)");
  System.out.println("       -b size      buffer size (between 1 and 64 KB, default is 16 KB)");
  System.out.println("       command      command (eg: ssh user@host)");
  System.exit(1);
  }
//...e
//...smain:2:
public static void main(String args[])
  {
  int i = 0;
  MapPortHostPort mapL = new MapPortHostPort();
  MapPortHostPort mapR = new MapPortHostPort();
  boolean child = false;
  String javaCommand = "java";
  String tunnelJar = "tunnel.jar";
  int buffsz = 0x4000;
  if ( args.length == 0 )
    // Pretty pointless to tunnel nothing, so show usage instead
    usage();
  while ( i < args.length && args[i].startsWith("-") )
    if ( args[i].equals("-child") )
      {
      child = true;
      ++i;
      }
    else if ( args[i].equals("-L") )
      {
      if ( ++i == args.length )
        usage();
      mapL.put(args[i++]);
      }
    else if ( args[i].equals("-R") )
      {
      if ( ++i == args.length )
        usage();
      mapR.put(args[i++]);
      }
    else if ( args[i].equals("-j") )
      {
      if ( ++i == args.length )
        usage();
      javaCommand = args[i++];
      }
    else if ( args[i].equals("-t") )
      {
      if ( ++i == args.length )
        usage();
      tunnelJar = args[i++];
      }
    else if ( args[i].equals("-b") )
      {
      if ( ++i == args.length )
        usage();
      try
        {
        buffsz = Integer.parseInt(args[i++]);
        }
      catch ( NumberFormatException e )
        {
        usage();
        }
      if ( buffsz < 1 || buffsz > 64 )
        fatal("buffer size should be between 1 and 64 KB");
      buffsz *= 1024;
      }
    else if ( args[i].equals("--") )
      // Allow the user command to be -L or -R
      // (yes, I know this is obscure, but I do like to be pure)
      {
      ++i;
      break;
      }
    else
      usage();
  if ( child )
    // We are the child
    // Respond to any -L from the parent in our Proxy
    // Start Listener for each -R
    {
    Textflow tf = new Textflow(System.in, System.out, true);
    Dictionary<Integer,Listener> listeners = mapR.startListeners(tf, buffsz);
    Proxy prx = new Proxy(mapL, tf, buffsz);
    tf.process(listeners, prx);
    }
  else
    // We are the parent, the rest of the args are the child process
    // Pass -L and -R information to the child
    // Start Listener for each -L
    // Respond to -R from the child in our Proxy
    {
    Vector<String> cmd = new Vector<String>();
    while ( i < args.length )
      cmd.add(args[i++]);
    cmd.add(javaCommand);
    cmd.add("-jar");
    cmd.add(tunnelJar);
    mapL.addMaps("-L", cmd);      
    mapR.addMaps("-R", cmd);      
    cmd.add("-child");
    try
      {
      Process p = Runtime.getRuntime().exec(cmd.toArray(new String[0]));
      Textflow tf = new Textflow(p.getInputStream(), p.getOutputStream(), false);
      Dictionary<Integer,Listener> listeners = mapL.startListeners(tf, buffsz);
      Proxy prx = new Proxy(mapR, tf, buffsz);
      tf.process(listeners, prx);
      }
    catch ( IOException e )
      {
      fatal(e.toString());
      }
    }
  }
//...e
  }
